@;***************************************************************************
@; Copyright 2000 Compaq Computer Corporation.                              *
@;                                           .                              *
@; Copying or modifying this code for any purpose is permitted,             *
@; provided that this copyright notice is preserved in its entirety         *
@; in all copies or modifications.  COMPAQ COMPUTER CORPORATION             *
@; MAKES NO WARRANTIES, EXPRESSED OR IMPLIED, AS TO THE USEFULNESS          *
@; OR CORRECTNESS OF THIS CODE OR ITS FITNESS FOR ANY PARTICULAR            *
@; PURPOSE.                                                                 *
@;***************************************************************************
@;
@; Copyright 2005, SDG Systems, LLC
@;

#define __ASSEMBLY__
#include "architecture.h"
#include "bootconfig.h"
#include "pxa.h"
#include "arm.h"
#include "boot_flags.h"

#undef __REG
#define __REG(x) x

@;
@; This macro is used to wait for a CP15 write and is needed
@; when we have to ensure that the last operation to the co-pro
@; was completed before continuing with operation.
@;
.macro  cpwait, rd
    mrc     p15, 0, \rd, c2, c0, 0          @; arbitrary read of cp15
    mov     \rd, \rd                        @; wait for completion
    sub     pc, pc, #4                      @; flush instruction pipeline
.endm

.macro  cpwait_ret, lr, rd
    mrc     p15, 0, \rd, c2, c0, 0          @; arbitrary read of cp15
    sub     pc, \lr, \rd, LSR #32           @; wait for completion and
                                            @; flush instruction pipeline
.endm

#define __BASE(x)   (x & 0xffff0000)
#define __OFFSET(x) (x & 0x0000ffff)

@; *************************
@; Start of executable code*
@; *************************

.globl _start
_start:
.globl ResetEntryPoint
ResetEntryPoint: @; 0x00:
        b       MainHiReset
.org 0x40
        .long 0x43454345
        .long 0x800cbd20

.org 0xf0
        .long 0xffffffff
        .long 0xffffffff
        .long 0x00000000
        .long 0x00415355

@; This is a hack to get the first word of the binary to have the correct value.
.org 0x1000, 0xFF
MainHiReset:

        @;  keep track of whether button was pushed in r12
        ldr     r12, boot_flags_init_val
        @;  BTW, r12 is sp

        @; Set up GPIOs
        ldr     r0, GPIO_BASE
        ldr     r1, GPSR0_Init_HX4700
        str     r1, [r0, #__OFFSET(GPSR0)]
        ldr     r1, GPSR1_Init_HX4700
        str     r1, [r0, #__OFFSET(GPSR1)]
        ldr     r1, GPSR2_Init_HX4700
        str     r1, [r0, #__OFFSET(GPSR2)]
        ldr     r1, GPSR3_Init_HX4700
        str     r1, [r0, #__OFFSET(GPSR3)]

        ldr     r1, GPCR0_Init_HX4700
        str     r1, [r0, #__OFFSET(GPCR0)]
        ldr     r1, GPCR1_Init_HX4700
        str     r1, [r0, #__OFFSET(GPCR1)]
        ldr     r1, GPCR2_Init_HX4700
        str     r1, [r0, #__OFFSET(GPCR2)]
        ldr     r1, GPCR3_Init_HX4700
        str     r1, [r0, #__OFFSET(GPCR3)]

        ldr     r1, GPDR0_Init_HX4700
        str     r1, [r0, #__OFFSET(GPDR0)]
        ldr     r1, GPDR1_Init_HX4700
        str     r1, [r0, #__OFFSET(GPDR1)]
        ldr     r1, GPDR2_Init_HX4700
        str     r1, [r0, #__OFFSET(GPDR2)]
        ldr     r1, GPDR3_Init_HX4700
        str     r1, [r0, #__OFFSET(GPDR3)]

        ldr     r1, GAFR0L_Init_HX4700
        str     r1, [r0, #__OFFSET(GAFR0_L)]
        ldr     r1, GAFR0U_Init_HX4700
        str     r1, [r0, #__OFFSET(GAFR0_U)]
        ldr     r1, GAFR1L_Init_HX4700
        str     r1, [r0, #__OFFSET(GAFR1_L)]
        ldr     r1, GAFR1U_Init_HX4700
        str     r1, [r0, #__OFFSET(GAFR1_U)]
        ldr     r1, GAFR2L_Init_HX4700
        str     r1, [r0, #__OFFSET(GAFR2_L)]
        ldr     r1, GAFR2U_Init_HX4700
        str     r1, [r0, #__OFFSET(GAFR2_U)]
        ldr     r1, GAFR3L_Init_HX4700
        str     r1, [r0, #__OFFSET(GAFR3_L)]
        ldr     r1, GAFR3U_Init_HX4700
        str     r1, [r0, #__OFFSET(GAFR3_U)]

        @;  let GPIOS be GPIOS
        ldr     r1, PMCR_BASE
        add     r2, r1,#__OFFSET(PSSR)
        mov     r0, #0x30
        str     r0, [r2]

@; Make sure the clock is on.
ldr     r1, CLOCK_BASE
ldr     r0, CKEN_INIT
str     r0, [r1,#__OFFSET(CKEN)]

check_for_action_button:
        bl      HX4700_CheckActionButton

MaybeForceBootldrStart:
        tst     r12, #BF_NORMAL_BOOT
        beq     BootldrStartForced

        @;  check to see whether this was a wake up from sleep
        ldr     r1, PMCR_BASE
        ldr     r0, [r1, #__OFFSET(RCSR)]
        and     r0, r0, #(RCSR_HWR + RCSR_GPR + RCSR_WDR + RCSR_SMR)

        @;  clear whatever status we found
        str     r0, [r1, #__OFFSET(RCSR)]

        teq     r0, #RCSR_SMR
        beq     WakeupStart

        b       BootldrStart

BootldrStartForced:
        @;  we got here cuz they held the action button dowm,
        bic     r12, r12, #BF_NORMAL_BOOT

BootldrStart:
        @; Turn on the UART, GPIO 32
        ldr r0, =0x40e0001c
        mov r1, #1
        str r1, [r0]

        bl      InitUARTFF

        @; check to see if we're operating out of DRAM
        bic     r4, pc, #0x000000FF
        bic     r4, r4, #0x0000FF00
        bic     r4, r4, #0x00FF0000
        cmp     r4, #DRAM_BASE0
        bne     RunningInFlash

RunningInDRAM:
        b       SKIP_SETUP

RunningInFlash:
        bl      SetupClocks

        bl      SetupStaticMemory
        bl      SetupDynamicMemory
        bl      TestMemory

SKIP_SETUP:

        bl      Relocate @; Always relocate to memory

UnlockFlash:
        @; First, set vppen
        ldr r0, =0x40e00020
        mov r3, #0x08000000
        str r3, [r0]

        mov r0, #0x100000       @; Start after 1 MB.
        mov r2, #0x08000000     @; Go until 0x08000000

1:
        ldr r1, =0x00600060
        str r1, [r0]
        ldr r1, =0x00d000d0
        str r1, [r0]
        ldr r1, =0x00ff00ff
        str r1, [r0]
        add r0, r0, #0x40000

        cmp r0, r2
        blt 1b

        @; Clear vppen
        ldr r0, =0x40e0002c
        str r3, [r0]

CALL_MAIN:

        @; pass boot flags as second arg
        orr     a2, r12, r12
        ldr     sp, DW_STACK_START
        mov     fp, #0           @; no previous frame, so fp=0

        @;
        @; check to see if this is a sleep reset.
        @; if so, then we want to enter the bootmenu directly.
        @; otherwise, we want to proceed as before, allowing
        @; a key or timeout to control the boot action
        @;
        tst     a2, #BF_SLEEP_RESET
        movne   a1, #1
        moveq   a1, #0


        bl      bootldr_main   @; call the main C entry point
        nop
        nop
        nop
        nop
        mov     pc, #FLASH_BASE @; reboot
        b       SKIP_SETUP

.ltorg



                                               @; 3322222222221111111111
                                               @; 10987654321098765432109876543210
                                               @;---------------------------------
GPDR0_Init_HX4700:           .long 0xda7a841c  @; 11011010011110101000010000011100

                                               @; 66665555555555444444444433333333
                                               @; 32109876543210987654321098765432
                                               @;---------------------------------
GPDR1_Init_HX4700:           .long 0x68efbf83  @; 01101000111011111011111110000011

                                               @; 99999988888888887777777777666666
                                               @; 54321098765432109876543210987654
                                               @;---------------------------------
GPDR2_Init_HX4700:           .long 0xbfbff7db  @; 10111111101111111111011111011011

                                               @; 1111111111111111111111111111
                                               @; 22222222111111111100000000009999
                                               @; 76543210987654321098765432109876
                                               @;---------------------------------
GPDR3_Init_HX4700:           .long 0x007ffff5  @; 00000000011111111111111111110101


                                              @; 1  1  1  1  1  1
                                              @; 5  4  3  2  1  0  9  8  7  6  5  4  3  2  1  0
GAFR0L_Init_HX4700:          .long 0x80115554 @; 10 00|00 00|00 01|00 00|00 00|00 01|01 00|01 00
                                              @; 3  3  2  2  2  2  2  2  2  2  2  2  1  1  1  1
                                              @; 1  0  9  8  7  6  5  4  3  2  1  0  9  8  7  6
GAFR0U_Init_HX4700:          .long 0x591a8558 @; 01 01|10 01|00 01|10 10|10 00|01 01|01 01|10 00
                                              @; 4  4  4  4  4  4  4  4  3  3  3  3  3  3  3  3
                                              @; 7  6  5  4  3  2  1  0  9  8  7  6  5  4  3  2
GAFR1L_Init_HX4700:          .long 0x600a9558 @; 01 10|00 00|00 00|10 10|10 01|01 01|01 01|10 00
                                              @; 6  6  6  6  5  5  5  5  5  5  5  5  5  5  4  4
                                              @; 3  2  1  0  9  8  7  6  5  4  3  2  1  0  9  8
GAFR1U_Init_HX4700:          .long 0x0005a0aa @; 00 00|00 00|00 00|01 01|10 10|00 00|10 10|10 10
                                              @; 7  7  7  7  7  7  7  7  7  7  6  6  6  6  6  6
                                              @; 9  8  7  6  5  4  3  2  1  0  9  8  7  6  5  4
GAFR2L_Init_HX4700:          .long 0xa0000000 @; 10 10|00 00|00 00|00 00|00 00|00 00|00 00|00 00
                                              @; 9  9  9  9  9  9  8  8  8  8  8  8  8  8  8  8
                                              @; 5  4  3  2  1  0  9  8  7  6  5  4  3  2  1  0
GAFR2U_Init_HX4700:          .long 0x00035402 @; 00 01|00 00|00 00|00 11|01 01|01 00|00 00|00 10
                                              @; 1  1  1  1  1  1  1  1  1  1  1  1
                                              @; 1  1  0  0  0  0  0  0  0  0  0  0  9  9  9  9
                                              @; 1  0  9  8  7  6  5  4  3  2  1  0  9  8  7  6
GAFR3L_Init_HX4700:          .long 0x00010000 @; 00 00|00 00|00 00|00 01|00 00|00 00|01 00|00 00
                                              @; 1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1
                                              @; 2  2  2  2  2  2  2  2  1  1  1  1  1  1  1  1
                                              @; 7  6  5  4  3  2  1  0  9  8  7  6  5  4  3  2
GAFR3U_Init_HX4700:          .long 0x00001404 @; 00 00|00 00|00 00|00 00|00 01|01 00|00 00|01 00

GPSR0_Init_HX4700:           .long 0x0935ede7
@; GPSR1_Init_HX4700:           .long 0xbbdf40f7    Want to turn off UART, GPIO 32, bit 0
GPSR1_Init_HX4700:           .long 0xbbdf40f6
GPSR2_Init_HX4700:           .long 0x0173c9f6
GPSR3_Init_HX4700:           .long 0x01f02342

GPCR0_Init_HX4700:           .long ~(0x0935ede7)
@; GPCR1_Init_HX4700:           .long ~(0xbbdf40f7)  Turning off UART
GPCR1_Init_HX4700:           .long ~(0xbbdf40f6)
GPCR2_Init_HX4700:           .long ~(0x0173c9f6)
GPCR3_Init_HX4700:           .long ~(0x01f02342)

boot_flags_init_val:
        .long   BF_NORMAL_BOOT

_bootldr_magic:
@; 0x20:
        .long   BOOTLDR_MAGIC       @; magic number so we can verify that we only put bootldr's into flash in the bootldr region

_bootldr_version:
@; 0x24:
        .long   BOOTLDR_VERSION

_bootldr_start:
@; 0x28:
        .long   _start              @; where this bootldr was linked, so we can put it in memory in the right place

_bootldr_architecture_magic:
@; 0x2C:
        .long   ARCHITECTURE_MAGIC  @; this contains the platform, cpu and machine id

_bootldr_capabilities:
@; 0x30
        @;
        @; bootldr capabilities
        @;
        .long   BOOTLDR_CAPS

_bootldr_signature:
@; 0x34
        @;
        @; bootldr sig
        @;
        .ascii  BL_SIG

_bootldr_end:
@; 0x38
        @; bootldr size
        .globl  __commands_end
        .long   __commands_end

HandleUndef:
        @; save all user registers
        adr     r13, AbortStorage
        stmia   r13, {r0-r12}
        mov     r12, r14
        bl      InitUARTFF
        ldr     r0,STR_UNDEF
        ldr     r1,SerBase
        BL      PrintWord
        mov     r0, r12
        ldr     r1,SerBase
        BL      PrintHexWord
        mrc     p15, 0, r0, c6, c0, 0 @; fault address
        ldr     r1,SerBase
        BL      PrintHexWord

        mov     r14, r12
        @; restore registers
        ldmia   r13, {r0-r12}
        @; now loop
1:      b 1b

HandleSWI:
        @; save all user registers
        adr     r13, AbortStorage
        stmia   r13, {r0-r12}
        mov     r12, r14
@;        bl      InitUARTFF
        ldr     r0,STR_SWI
        ldr     r1,SerBase
        BL      PrintWord
        mov     r0, r12
        ldr     r1,SerBase
        BL      PrintHexWord
        mrc     p15, 0, r0, c6, c0, 0 @; fault address
        ldr     r1,SerBase
        BL      PrintHexWord

        mov     r14, r12
        @; restore registers
        ldmia   r13, {r0-r12}
        @; now loop
1:      b 1b

HandlePrefetchAbort:
        @; save all user registers
#if 0
        adr     r13, AbortStorage
        stmia   r13, {r0-r12}
        mov     r12, r14
        bl      InitUARTFF
#endif
        ldr     r0,STR_PREFETCH_ABORT
        ldr     r1,SerBase
        BL      PrintWord
        mov     r0, r14
        ldr     r1,SerBase
        BL      PrintHexWord
        mov     r0, r13
        ldr     r1,SerBase
        BL      PrintHexWord
        mov     r0, r12
        ldr     r1,SerBase
        BL      PrintHexWord
#if 0
        mrc     p15, 0, r0, c6, c0, 0 @; fault address
        ldr     r1,SerBase
        BL      PrintHexWord

        mov     r14, r12
        @; restore registers
        ldmia   r13, {r0-r12}
#endif
        @; now loop
1:      b 1b

AbortStorage:
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0

HandleDataAbort:
        @; save all user registers
        adr     r13, AbortStorage
        stmia   r13, {r0-r12}
        mov     r12, r14
@;        bl      InitUARTFF
        ldr     r0,STR_DATA_ABORT
        mov     r1, #0x40000000
        orr     r1, r1, #0x00100000
        BL      PrintWord
        mov     r0, r12
        ldr     r1,SerBase
        BL      PrintHexWord
        mrc     p15, 0, r0, c6, c0, 0 @; fault address
        ldr     r1,SerBase
        BL      PrintHexWord

        mov     r14, r12
        @; restore registers
        ldmia   r13, {r0-r12}
        @; now loop
1:      b 1b

HandleIRQ:
@;      b       MainHiReset

        @; save all user registers
        adr     r13, AbortStorage
        stmia   r13, {r0-r12}
        mov     r12, r14
        bl      InitUARTFF
        ldr     r0,STR_IRQ
        ldr     r1,SerBase
        BL      PrintWord
        mov     r0, r12
        ldr     r1,SerBase
        BL      PrintHexWord

        mov     r14, r12
        @; restore registers
        ldmia   r13, {r0-r12}
        @; now loop
1:      b 1b

HandleFIQ:
        @; save all user registers
        adr     r13, AbortStorage
        stmia   r13, {r0-r12}
        mov     r12, r14
        bl      InitUARTFF
        ldr     r0,STR_FIQ
        ldr     r1,SerBase
        BL      PrintWord
        mov     r0, r12
        ldr     r1,SerBase
        BL      PrintHexWord

        mov     r14, r12
        @; restore registers
        ldmia   r13, {r0-r12}
        @; now loop
1:      b 1b

HandleNotUsed:
        @; save all user registers
        adr     r13, AbortStorage
        stmia   r13, {r0-r12}
        mov     r12, r14
        bl      InitUARTFF
        ldr     r0,STR_NOT_USED
        ldr     r1,SerBase
        BL      PrintWord
        mov     r0, r12
        ldr     r1,SerBase
        BL      PrintHexWord

        mov     r14, r12
        @; restore registers
        ldmia   r13, {r0-r12}
        @; now loop
1:      b 1b

@;      Reset entry point




@;*********************************************************
@;**  Start of the Wake Up Code and Power Detection      **
@;*********************************************************
WakeupStart:
        @; get static memory going again
        bl      SetupStaticMemory

#if 0
@; Turn on the UART, GPIO 32
ldr r0, =0x40e0001c
mov r1, #1
str r1, [r0]

bl      InitUARTFF

ldr     r0, STR_NEWLINES
ldr     r1, SerBase
bl      PrintWord
ldr     r0, STR_WAKE
ldr     r1, SerBase
bl      PrintWord
#endif

@; Ensure that asic3 is not reset
@;ldr r0, =0x40e00020
@;mov r1, #0x80
@;ldr r1, [r0]



        @;  This is what we need to do for wakeup.
        @;          1. Check to see if ASIC3 woke us up.  If it didn't, then we do
        @;             a normal resume.
        @;          2. If ASIC3 woke us up,
        @;               A. check the Top Level Interrupt Status register to see
        @;                  if it was an LED.  If so, do general power state
        @;                  detect, and read battery state from one wire to see if
        @;                  it is done charging.  If so, turn off LEDs.
        @;               B. check GPIOD IntStatus register
        @;                  If it was AC rising or falling, do general power state
        @;                    detect and go back to sleep.
        @;                  If it was USB rising (removal), do general power state
        @;                    detect and go back to sleep.
        @;                  If it was USB falling (insert), do normal wake-up
        @;                  If it was none of the above, do a normal wakeup.

        @; Check why we were awakened
        ldr r3, =0x40f00018
        ldr r4, [r3]
        @; In this test, we look at the edge detect for OS Timer
        tst r4, #(1 << 30)      @; EDP1
        bne check_led_wakeup    @; branch if it's set
        @; In this test, we look at the edge detect for ASIC 3
        tst r4, #(1 << 12)      @; GPIO12 is ASIC3
        beq normal_resume       @; branch if it's not set

check_asic3:
        @; we were awakened by ASIC3, so let's examine the top level interrupt status
        ldr r3, =0x0c000582     @; ASIC3 INTR STATUS
        ldrh r4, [r3]

        tst r4, #(1 << 3)       @; Check if it was GPIOD that woke us up.  If so, then
                                @; there was probably a power change.  For USB
                                @; change, we wake up all the way only if it
                                @; was plugged. Unplugged doesn't.


        @; OK. We had a GPIOD interrupt.  Let's figure out which one.
        ldr r3, =0x0c000192     @; GPIOD Int Status
        ldrh r4, [r3]
        mov r0, #0
        strh r0, [r3]           @; write zero to clear

        tst r4, #(1 << 9)       @; AC
        bne general_power_evaluation           @; It was set, we had an AC event

        tst r4, #(1 << 14)      @; USB
        bne general_power_evaluation           @; It was set, we had a USB event

        b normal_resume

general_power_evaluation:
        mov r0, #16     @; To minimize bounce
        bl Smsleep

        @; If GPIOD, bit 14 is 0, then USB is inserted.
        @; If GPIOD, bit 14 is 1, then USB is not inserted.
        @; If GPIOD, bit  9 is 0, then AC is inserted.
        @; If GPIOD, bit  9 is 1, then AC is not inserted.
        ldr r3, =0x0c000198
        ldrh r4, [r3]

        @; r5 keeps track of whether we are AC or USB
        tst r4, #(1 << 9)   @; AC
        moveq r5, #1
        beq ac_is_plugged_in        @; branch if bit is zero
        tst r4, #(1 << 14)  @; USB
        moveq r5, #3
        beq ac_is_plugged_in        @; branch if bit is zero
@;ldr r1,SerBase
@;mov r0, #'a'
@;bl PrintChar

        @; AC is not plugged in.  In this case, we turn the LED off.
        @; Turn off LED0
        ldr r0, =0x0c000380
        ldr r1, =0x40
        strh r1, [r0]

        ldr r0, =0x0c000400
        ldr r1, =0x40
        strh r1, [r0]

        @; Make the interrupts falling edge triggered now
        ldr r0, =0x0c000186
        ldrh r1, [r0]
        orr r1, r1, #(1 << 9)   @; AC
        orr r1, r1, #(1 << 14)  @; USB
        strh r1, [r0]

        add r0, r0, #2
        ldrh r1, [r0]
        bic r1, r1, #(1 << 9)   @; AC
        bic r1, r1, #(1 << 14)  @; USB
        strh r1, [r0]

        @; Unmask the ASIC3 GPIO interrupt
        ldr r0, =0x0c000580
        ldrh r1, [r0]
        orr r1, r1, #(1 << 0)
        strh r1, [r0]

        @; In this case, we don't want to wake up again from the timer, so we
        @; set the PSLR to 0.
        ldr r0, =0x40f00034
        ldr r1, [r0]
        bic r1, r1, #(3 << 2)
        str r1, [r0]

        @; Now clear bit 30 of the PWER to enable Power Island (PI, ostimer) wakeups.
        ldr r0, =0x40f0000c
        ldr r1, [r0]
        bic r1, r1, #(1 << 30)      @; WEP1
        orr r1, r1, #(1 << 12)      @; Always want ASIC3 GPIO, too.
        str r1, [r0]

        @; Turn off the timer
        ldr r0, =0x40a000c0
        mov r1, #0x00
        str r1, [r0]

        ldr r0, =0x40a00014
        mov r1, #0x10
        str r1, [r0]

        mov r0, #0
        b go_back_to_sleep

ac_is_plugged_in:

@;ldr r1,SerBase
@;mov r0, #'A'
@;bl PrintChar

@;mov r0, r5
@;ldr r1,SerBase
@;bl PrintHexWord

        @; Make the interrupt rising edge triggered now
        ldr r0, =0x0c000186
        ldrh r1, [r0]
        orr r1, r1, #(1 << 9)   @; AC
        orr r1, r1, #(1 << 14)  @; USB
        strh r1, [r0]

        add r0, r0, #2
        ldrh r1, [r0]
        orr r1, r1, #(1 << 9)   @; AC
        orr r1, r1, #(1 << 14)  @; USB
        strh r1, [r0]

        @;   Set USB_CHARGE_RATE, GPIO 96 based on whether USB is plugged in.
        tst r5, #(1 << 1)       @; r5[1] == '1' when USB
        ldrne r3, =0x40e00118   @; Set   the GPIO bit when bit 1 is set
        ldreq r3, =0x40e00124   @; Clear the GPIO bit when bit 1 is clear
        ldr r4, =(1 << 0)       @; Set bit zero of GPIO_xxx_120:96
        str r4, [r3]
        @;   Clear CHARGE_EN, GPIO 72  (turn charge on)
        ldr r3, =0x40e0002c
        ldr r4, =(1 << 8)       @; Set bit 8 of GPIO_CLEAR_95:64
        str r4, [r3]

        @; Let the battery figure out its parameters
        mov r0, #88     @; takes 88 ms to update current
        bl Smsleep

        mov r11, r5     @; saving r5, the AC/USB state

        @; Now check to see if the charge is complete or not
        bl battery_current

@;mov r5, r0
@;ldr r1, SerBase
@;bl PrintHexWord
@;mov r0, r5

        mov r5, r11     @; restoring r5

        @; Now check our state stored in the (unused) LED count register
        ldr r1, =0x0c000386
        ldrh r2, [r1]
        cmp r2, #0
        moveq r2, #75   @; If it's zero, then we have not reached complete yet,
        movne r2, #150  @; If it's not zero, then we have not reached and we
                        @; don't want to blink until it's above this.

        cmp r0, r2
        blt charge_complete

        @; r1 still points to LED count register
        @; Set 0x0c000386 to zero
        mov r2, #0
        strh r2, [r1]    @; clear it out so we have to get to a smaller current
                         @; before saying it's charged


        @; Turn on LED0.
        ldr r0, =0x0c000380
        ldrh r1, [r0]
        cmp r1, #16
        beq led_already_setup
        ldr r1, =0x16
        strh r1, [r0]

        ldr r1, =0x100
        add r0, r0, #2
        strh r1, [r0]

        ldr r1, =0x80
        add r0, r0, #2
        strh r1, [r0]

led_already_setup:
        @; In this case, we do want to wake up again by the timer, so we set
        @; the PSLR to (2 << 2) to keep the OS timers going.
        ldr r0, =0x40f00034
        ldr r1, [r2]
        orr r1, r1, #(2 << 2)
        str r1, [r0]

        @; Now set bit 30 of the PWER to enable Power Island (PI, ostimer) wakeups.
        ldr r0, =0x40f0000c
        ldr r1, [r0]
        orr r1, r1, #(1 << 30)      @; WEP1
        orr r1, r1, #(1 << 12)      @; Always want ASIC3 GPIO, too.
        str r1, [r0]

        @; Configure the OS timer to wake us up
        ldr r0, =0x40a000c0
        ldr r1, =0x4b           @; Periodic, self-resetting, 1-second timer
        str r1, [r0]

        sub r0, r0, #0x40       @; r0 = 0x40a00080
        ldr r1, =5              @; wake up every x seconds.
        str r1, [r0]

        ldr r0, =0x40a0001c     @; Interrupt enable register
        ldr r1, =0x10           @; timer 4
        str r1, [r0]

        ldr r0, =0x40a00040     @; Start the timer
        ldr r1, =0x00
        str r1, [r0]

        mov r0, r5              @; r5 was set way back to either 1 or 3 based on GPIO states
        b go_back_to_sleep

go_back_to_sleep:
@; input:  r0 == 0, no charge
@;         r0 & 1, Charge Enable
@;         r0 & 2, USB Charge Rate

        @; These values are reset on sleep exit.
        ldr r2, =0x080DC01C
        ldr r1, =0x40f00020     @; PGSR0
        str r2, [r1]

        ldr r2, =0x34CF0002
        add r1, r1, #4          @; PGSR1
        str r2, [r1]

        ldr r2, =0x0123C18C     @; Bit 8 is set, charge disabled, by default
@;        ldr r2, =0x0123C10C     @; Bit 8 is set, charge disabled, by default, ASIC3 reset
        cmp r0, #0
        bicne r2, r2, #(1 << 8) @; Clear the bit (turn charge on) if r0[0] is set
        add r1, r1, #4          @; PGSR2
        str r2, [r1]

        ldr r2, =0x00100202     @; Bit 0 is not set, USB_CHARGE_RATE off, by default
        tst r0, #2
        orrne r2, r2, #(1 << 0) @; Turn on USB charge rate if r0[2] is set
        add r1, r1, #4          @; PGSR3
        str r2, [r1]

@; @; Print some information about ASIC3
@; ldr r1, SerBase         @; the value of r0
@; bl PrintHexWord
@; 
@; ldr r1,SerBase
@; mov r0, #'I'
@; bl PrintChar
@; ldr r1,SerBase
@; mov r0, #'\r'
@; bl PrintChar
@; ldr r1,SerBase
@; mov r0, #'\n'
@; bl PrintChar
@; 
@; ldr r0, =0x0c000186    @; interrupt edge/level
@; ldrh r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x0c000188     @; ASIC3 falling/rising edge
@; ldrh r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x0c000380     @; ASIC3 LED0
@; ldrh r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x0c000580     @; ASIC3 Int Mask
@; ldrh r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x0c000582     @; ASIC3 Int Status
@; ldrh r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x40f00034     @; PSLR
@; ldr r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x40f0000c     @; PWER
@; ldr r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x40a000c0     @; OS Timer Config
@; ldr r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x40a0001c     @; OS Timer IRQ Enable
@; ldr r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x40a00014     @; OS Timer IRQ Status  should be zero
@; ldr r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x40f00028     @; PGSR2
@; ldr r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =0x40f0002c     @; PGSR3
@; ldr r0, [r0]
@; ldr r1, SerBase
@; bl PrintHexWord
@; 
@; ldr r0, =300
@; bl Smsleep

        @; always check GPIOD interrupt again right before we sleep.
        ldr r0, =0x0c000192     @; GPIOD Int Status
        ldr r0, [r0]
        cmp r0, #0
        bne check_asic3

        mov r0, #3
        mcr p14, 0, r0, c7, c0, 0




check_led_wakeup:

@; ldr r1,SerBase
@; mov r0, #'L'
@; bl PrintChar

        @; Clear the timer interrupt source
        ldr r0, =0x40a00014
        ldr r1, =0x10
        str r1, [r0]

        b general_power_evaluation

charge_complete:
        @; OK, we're done charging.  Turn on the LED and go back to sleep.
        @; Turn on LED0
        ldr r0, =0x0c000400
        ldr r1, =0x56
        strh r1, [r0]

        ldr r1, =0x10
        add r0, r0, #2
        strh r1, [r0]

        ldr r1, =0x10
        add r0, r0, #2
        strh r1, [r0]

        ldr r0, =0x0c000380
        ldr r1, =0x40
        strh r1, [r0]

        @; r0 becomes 0x0c000386
        add r0, r0, #6          @; Keep some state here.  Set this register to 1
        ldr r1, =0x1            @; to indicate that we have fully charged.  This is so
        strh r1, [r0]           @; we can have a hysteresis.

        @; In this case, we don't want to wake up again, so we set the PSLR to 0.
        ldr r0, =0x40f00034
        ldr r1, [r0]
        bic r1, r1, #(3 << 2)
        str r1, [r0]

        @; . . . and clear the Power Island wake up source in PWER
        ldr r0, =0x40f0000c
        ldr r1, [r0]
        bic r1, r1, #(1 << 30)      @; WEP1
        orr r1, r1, #(1 << 12)      @; Always want ASIC3 GPIO
        str r1, [r0]

        @; we leave charge_en charging
        mov r0, #3  @; use USB charge which is safe in all cases.
        b go_back_to_sleep


normal_resume:

        @; get SDRAM going again
        bl      SetupDynamicMemory

        @; retrieve the scratchpad value and jump to that address
        ldr     r7, PMCR_BASE
        ldr     r6, [r7, #__OFFSET(PSPR)]
        mov     pc, r6
@;*********************************************************
@;**  End of the Wake Up Code and Power Detection        **
@;*********************************************************


@; ********************************************************************
@; SetupClocks -- inits the clock registers
@;
@;   modifies r0,r1
@;
@; ********************************************************************

SetupClocks:
        @;  this initializes the clocks to valid values
        ldr     r1, CLOCK_BASE
        ldr     r0, CCCR_INIT
        str     r0, [r1,#__OFFSET(CCCR)]
        ldr     r0, CKEN_INIT
        str     r0, [r1,#__OFFSET(CKEN)]
        ldr     r0, OSCC_INIT
        str     r0, [r1,#__OFFSET(OSCC)]

        mrc     p14, 0, r0, c6, c0, 0
        orr     r0, r0, #0xb           @;  select turbo mode, fast bus, start FCS
        mcr     p14, 0, r0, c6, c0, 0
        mov     pc, lr

@; ********************************************************************
@; SetupXxxxMemory -- inits the SDRAM and MSC registers
@;
@;   modifies r0,r1
@;
@; ********************************************************************
SetupStaticMemory:
        mov     r6,lr

        @; This procedure is outlined in the PXA27x Developer's Manual, page 6-41.

        @; Step 0:       print out step 0
@;        ldr     r0,STR_STEP
@;        ldr     r3,=0xffffff00
@;        and     r0,r0,r3
@;        orr     r0,r0,#'0'
@;        ldr     r1,SerBase
@;        bl      PrintWord

        @;  Step 1:      wait 200 usec for the internal clocks to stabilize.
        ldr     r0,OSCR_BASE
        mov     r1,#0
        str     r1,[r0,#__OFFSET(OSCR)]
        ldr     r3,=0xa00 @;  200 usec is 0x2e1 so this should be fine
10:
        ldr     r2,[r0,#__OFFSET(OSCR)]
        cmp     r3,r2
        bgt     10b

@;        ldr     r0,STR_STEP
@;        ldr     r3,=0xffffff00
@;        and     r0,r0,r3
@;        orr     r0,r0,#'1'
@;        ldr     r1,SerBase
@;        bl      PrintWord

        @;  Step 1a: Set up the MSC registers
        @; MSC0 controls the accesses to CS0 and CS1 where the flash resides
        ldr     r0, MEM_BASE
        ldr     r1, =0x25e225e2 @; MSC0_INIT
            @; 0x25e225e2    HTC: 0x24c224c2
            @;  RBUF = 0        Return Data Buffer vs Streaming Behavior
            @;  RRR  = 2        ROM Recovery Time
            @;  RDN  = 5        ROM Delay Next Access
            @;  RDF  = 0xe      ROM Delay First Access
            @;  RBW  = 0        ROM Bus Width  0 = 32, 1 = 16
            @;  RT   = 2        ROM Type 2 = Burst of four ROM/Flash
        str     r1, [r0,#__OFFSET(MSC0)] @;  spec says to ldr the mscs after a set
        ldr     r2, [r0,#__OFFSET(MSC0)]

        @; MSC1 controls the accesses to CS2 and CS3
        ldr     r1, =0x12cc2364 @; MSC1_INIT
            @;               HTC: 0x026c1444
            @; 0x2364
            @; CS2 is the ATI W3220
            @;  RBUF = 0        Return Data Buffer vs Streaming Behavior
            @;  RRR  = 2        ROM Recovery Time
            @;  RDN  = 3        ROM Delay Next Access
            @;  RDF  = 0x6      ROM Delay First Access
            @;  RBW  = 0        ROM Bus Width  0 = 32, 1 = 16
            @;  RT   = 4        ROM Type 4 = VLIO

            @; 0x12cc
            @; CS3 is the ASIC3
            @;  RBUF = 0        Return Data Buffer vs Streaming Behavior
            @;  RRR  = 1        ROM Recovery Time
            @;  RDN  = 2        ROM Delay Next Access
            @;  RDF  = 0xc      ROM Delay First Access
            @;  RBW  = 1        ROM Bus Width  0 = 32, 1 = 16
            @;  RT   = 4        ROM Type 4 = VLIO
        str     r1, [r0,#__OFFSET(MSC1)] @;  spec says to ldr the mscs after a set
        ldr     r2, [r0,#__OFFSET(MSC1)]

        @; MSC2 controls the accesses to CS4 and CS5
        ldr     r1, =0x16dc7ffc
            @;                  HTC: 0x03ac7ffc
            @; 0x7ffc
            @; CS4 is not used
            @;  RBUF = 0        Return Data Buffer vs Streaming Behavior
            @;  RRR  = 7        ROM Recovery Time
            @;  RDN  = f        ROM Delay Next Access
            @;  RDF  = 0xf      ROM Delay First Access
            @;  RBW  = 1        ROM Bus Width  0 = 32, 1 = 16
            @;  RT   = 4        ROM Type 4 = VLIO

            @; 0x16dc
            @; CS5 is the TI WLAN, External UART
            @;  RBUF = 0        Return Data Buffer vs Streaming Behavior
            @;  RRR  = 1        ROM Recovery Time
            @;  RDN  = 6        ROM Delay Next Access
            @;  RDF  = 0xd      ROM Delay First Access
            @;  RBW  = 1        ROM Bus Width  0 = 32, 1 = 16
            @;  RT   = 4        ROM Type 4 = VLIO
        str     r1, [r0,#__OFFSET(MSC2)] @;  spec says to ldr the mscs after a set
        ldr     r2, [r0,#__OFFSET(MSC2)]

        mov     pc, r6


SetupDynamicMemory:
        mov     r6, lr

@;        ldr     r0, STR_STEP
@;        ldr     r3, =0xffffff00
@;        and     r0, r0,r3
@;        orr     r0, r0,#'a'
@;        ldr     r1, SerBase
@;        bl      PrintWord

        @; HTC: Programs buffer strengh control to all 0xf.


        @;  Step 1b:     configure the PCMCIA card using  mecr, mcmem0,mcatt0,mcatt1,mcio0,mcio133
        @;  we'll do this later....
@;        ldr     r0,STR_STEP
@;        ldr     r3,=0xffffff00
@;        and     r0,r0,r3
@;        orr     r0,r0,#'b'
@;        ldr     r1,SerBase
@;        bl      PrintWord

        @;  Step 1c:  configure FLYCNFG
        @; It can reset to anything, but we're not doing fly by DMA in the bootldr
@;        ldr     r0,STR_STEP
@;        ldr     r3,=0xffffff00
@;        and     r0,r0,r3
@;        orr     r0,r0,#'c'
@;        ldr     r1,SerBase
@;        bl      PrintWord

        @; Step 1d: Configure but do not enable SDRAMs.
        ldr     r1, MDCNFG_STEP_1d_INIT
        str     r1, [r0,#__OFFSET(MDCNFG)]

@;        ldr     r0,STR_STEP
@;        ldr     r3,=0xffffff00
@;        and     r0,r0,r3
@;        orr     r0,r0,#'d'
@;        ldr     r1,SerBase
@;        bl      PrintWord

        @;  Step 1e: initial configure of MDREFR
        @; For PXA27x, we need to keep the values of APD (clear) and SLFRSH (set)
        ldr     r0, MEM_BASE
        ldr     r1, MDREFR_STEP_5_INIT_A
        @; HTC sets DRI to 0x19
        str     r1, [r0,#__OFFSET(MDREFR)]

@;        ldr     r0,STR_STEP
@;        ldr     r3,=0xffffff00
@;        and     r0,r0,r3
@;        orr     r0,r0,#'e'
@;        ldr     r1,SerBase
@;        bl      PrintWord

        @; Step 2: Synchronous static memory -- we dont have any
        @; I believe we need to program the flash here to synchronous mode: 0x25c2
        @; mov r0, #0x0
        @; orr r0, r0, #0x2500
        @; orr r0, r0, #0x00c2
        @; mov r1, #0x00600060 @; both banks at once
        @; str r1, [r0]    @; the data is in the address
        @; mov r1, #0x00030003
        @; str r1, [r0]
        @;
        @; OK. Here's a thought.  How can we change the flash reads from
        @; asynchronous (the reset setting) to synchronous while we're running
        @; from that flash.  I believe the answer is that you can't and that's why
        @; the HTC bootloader doesn't touch SXCNFG.  You probably have to copy
        @; yourself to RAM before you can change the flash to synchronous.
        @; So I'm removing this.
@;        ldr     r0, MEM_BASE
@;        ldr     r1, =0x00002013 @; SXCNFG_INIT
            @; 0x00002013
            @;   SXTP0 = 2      SX Memory Type, 2 = synchronous flash, burst-of-eight
            @;   SXCL0 = 4      5 clocks == code 4
            @;   SXEN1 = 1      Enable
            @;   SXEN0 = 1      Enable
@;        str     r1, [r0, #__OFFSET(SXCNFG)]

@;        ldr     r0, STR_STEP
@;        ldr     r3, =0xffffff00
@;        and     r0, r0,r3
@;        orr     r0, r0,#'2'
@;        ldr     r1, SerBase
@;        bl      PrintWord

        @; Step 3:      SDRAM again
        ldr     r0, MEM_BASE
        @;  MDREFR self refresh turned off
        ldr     r1, [r0,#__OFFSET(MDREFR)]
        orr     r1, r1,#MDREFR_K1RUN
        orr     r1, r1,#MDREFR_K1DB2    @; turn this off if we don't do bit 25 in CCCR
        str     r1, [r0,#__OFFSET(MDREFR)] @; first state transition to Self-Refresh
        bic     r1, r1,#MDREFR_SLFRSH
        str     r1, [r0,#__OFFSET(MDREFR)] @; second transition to Power-Down
        orr     r1, r1,#MDREFR_E1PIN
        str     r1, [r0,#__OFFSET(MDREFR)] @; third transition to PWRDNX

@;        ldr     r0, STR_STEP
@;        ldr     r3, =0xffffff00
@;        and     r0, r0,r3
@;        orr     r0, r0,#'3'
@;        ldr     r1, SerBase
@;        bl      PrintWord

        @; Step 4: Same as 1d???
        @;  MDCONFIG setup up but not enabled
        @; haret: 0x0ad00ac9
        @; Set bits: 27 25 23 22 20 11 9 7 6 3 0
        @; SETALWAYS27, DTC2=2, DNB2, DRAC2=2, DCAC=2, SETALWAYS11, DTC0=2, DNB0, DRAC0=1, DCAC0=1, DE0
        @; We always write the set-always bits
        ldr     r0, MEM_BASE
        ldr     r1, MDCNFG_STEP_1d_INIT
        str     r1, [r0,#__OFFSET(MDCNFG)]

@;        ldr     r0, STR_STEP
@;        ldr     r3, =0xffffff00
@;        and     r0, r0,r3
@;        orr     r0, r0,#'4'
@;        ldr     r1, SerBase
@;        bl      PrintWord

        @; Step 5:      wait for the sdclocks to stabilize (200 usec)
        ldr     r0, OSCR_BASE
        mov     r1, #0
        str     r1, [r0,#__OFFSET(OSCR)]
        ldr     r3, =0xa00 @;  200 usec is x2e1 so this should be fine
20:
        ldr     r2, [r0,#__OFFSET(OSCR)]
        cmp     r3, r2
        bgt     20b

@;        ldr     r0, STR_STEP
@;        ldr     r3, =0xffffff00
@;        and     r0, r0,r3
@;        orr     r0, r0,#'5'
@;        ldr     r1, SerBase
@;        bl      PrintWord

        @; Step 6:      Not necessary
@;        ldr     r0, STR_STEP
@;        ldr     r3, =0xffffff00
@;        and     r0, r0,r3
@;        orr     r0, r0,#'6'
@;        ldr     r1, SerBase
@;        bl      PrintWord

        @; Step 7:      trigger refreshes by reading sdram
        mov     r0, #0xa0000000
        ldr     r3, =0x200
        mov     r2, #0
20:
        ldr     r1, [r0]
        add     r2, r2,#1
        cmp     r3, r2
        bgt     20b

        mov     r0, #0xa2000000
        ldr     r3, =0x200
        mov     r2, #0
20:
        ldr     r1, [r0]
        add     r2, r2,#1
        cmp     r3, r2
        bgt     20b

@;        ldr     r0, STR_STEP
@;        ldr     r3, =0xffffff00
@;        and     r0, r0,r3
@;        orr     r0, r0,#'7'
@;        ldr     r1, SerBase
@;        bl      PrintWord

        @; Step 8 not necessary
@;        ldr     r0, STR_STEP
@;        ldr     r3, =0xffffff00
@;        and     r0, r0,r3
@;        orr     r0, r0,#'8'
@;        ldr     r1, SerBase
@;        bl      PrintWord

        @; Step 9:     enable the sdram in the mdconfig register
        ldr     r0, MEM_BASE
        ldr     r1, [r0, #__OFFSET(MDCNFG)]
        orr     r1, r1, #MDCNFG_DE0
@;        orr     r1, r1, #MDCNFG_DE1
        str     r1, [r0, #__OFFSET(MDCNFG)]

@;        ldr     r0, STR_STEP
@;        ldr     r3, =0xffffff00
@;        and     r0, r0, r3
@;        orr     r0, r0, #'9'
@;        ldr     r1, SerBase
@;        bl      PrintWord

        @; Step 10:     @;  write out the register mode offset value to the sdram
        ldr     r0, MEM_BASE
        mov     r1, #0
        str     r1, [r0,#__OFFSET(MDMRS)]

@;        ldr     r0, STR_STEP
@;        ldr     r3, =0xffffff00
@;        and     r0, r0,r3
@;        orr     r0, r0,#'A'
@;        ldr     r1, SerBase
@;        bl      PrintWord

        @; HTC: Enable auto power down

        mov     pc, r6


@; ********************************************************************
@; TestMemory -- do we have dram?
@;
@;   modifies r0,r1,r2,r3,r4,
@;
@;      r5 -- dram_ptr
@;      r6 lr
@;      r7 -- when to print ptr
@;      r8 -- value
@;      r9 -- saved value
@;      r10 -- loop end
@;      r11 cmp value
@;
@; ********************************************************************
TestMemory:
        mov     r6,lr
@;        ldr     r0, STR_MTST
@;        ldr     r1, SerBase
@;        bl      PrintWord

TestDramBank0:
        @; some test code
        mov     r5, #DRAM_BASE0
        mov     r7, #0x00000001
TestDramBank0Loop:
        mov     r0, r7
        ldr     r2,[r5, #48]
        str     r0, [r5, #48]
        ldr     r0, [r5, #48]
        str     r2, [r5, #48]
@;        ldr     r1, SerBase
@;        bl      PrintHexWord
        tst     r7, #0x80000000
        bne     TestDramBank0Done
        mov     r7, r7, lsl #1
        b       TestDramBank0Loop
TestDramBank0Done:


        @;  randomish number
        mov     r4,#0xa0000000
        ldr     r5,[r4]
        ldr     r1, =0xdeadbeef
        str     r1,[r4]
        mov     r0,r4
@;        ldr     r1,SerBase
@;        bl      PrintHexWord
        mov     r4,#0xa0000000
        ldr     r0,[r4]
@;        ldr     r1,SerBase
@;        bl      PrintHexWord
        mov     r4,#0xa0000000
        str     r5,[r4]

        @; XXX We should actually have a test here!!  Not just print stuff.


@;        ldr     r0,MEM_BASE
@;        add     r0,r0,#__OFFSET(MDCNFG)
@;        ldr     r1,SerBase
@;        bl      PrintHexWord
@;        ldr     r4,MEM_BASE
@;        ldr     r0,[r4,#__OFFSET(MDCNFG)]
@;        ldr     r1,SerBase
@;        bl      PrintHexWord

@;        ldr     r0,MEM_BASE
@;        add     r0,r0,#__OFFSET(MDREFR)
@;        ldr     r1,SerBase
@;        bl      PrintHexWord
@;        ldr     r4,MEM_BASE
@;        ldr     r0,[r4,#__OFFSET(MDREFR)]
@;        ldr     r1,SerBase
@;        bl      PrintHexWord

@;        ldr     r0,MEM_BASE
@;        add     r0,r0,#__OFFSET(MSC0)
@;        ldr     r1,SerBase
@;        bl      PrintHexWord
@;        ldr     r4,MEM_BASE
@;        ldr     r0,[r4,#__OFFSET(MSC0)]
@;        ldr     r1,SerBase
@;        bl      PrintHexWord

@;        ldr     r0,MEM_BASE
@;        add     r0,r0,#__OFFSET(MSC1)
@;        ldr     r1,SerBase
@;        bl      PrintHexWord
@;        ldr     r4,MEM_BASE
@;        ldr     r0,[r4,#__OFFSET(MSC1)]
@;        ldr     r1,SerBase
@;        bl      PrintHexWord

@;        ldr     r0,MEM_BASE
@;        add     r0,r0,#__OFFSET(MSC2)
@;        ldr     r1,SerBase
@;        bl      PrintHexWord
@;        ldr     r4,MEM_BASE
@;        ldr     r0,[r4,#__OFFSET(MSC2)]
@;        ldr     r1,SerBase
@;        bl      PrintHexWord

        @; debug print
DebugPrintAfterTestDram:
@;        ldr     r0, STR_ENDM
@;        ldr     r1, SerBase
@;        bl      PrintWord

        mov     pc, r6

Relocate:
        mov r6, lr

@;        ldr     r0,STR_RELO
@;        ldr     r1,SerBase
@;        bl      PrintWord

        ldr     r8, =reference
        ldr     r9, _bootldr_start
        sub     r8, r8, r9
        mov     r9, pc
        sub     r9, r9, r8
reference:

@;        @; Print copy from address
@;        ldr     r0,STR_FROM
@;        ldr     r1,SerBase
@;        bl      PrintWord
@;        mov     r0, r9
@;        ldr     r1, SerBase
@;        bl      PrintHexWord


        ldr     r5, _bootldr_start

@;        @; Print copy to address
@;        ldr     r0,STR_TO
@;        ldr     r1,SerBase
@;        bl      PrintWord
@;        mov     r0, r5
@;        ldr     r1, SerBase
@;        bl      PrintHexWord

        ldr     r7, _bootldr_end
        @; r9 is where we are currently running from
        @; r5 has link-time start address of bootldr (where we're going)
        @; r7 gets link-time end address of bootldr (the end of where we're going)
        @; Save r9
        mov     r1, r9
1:      ldr     r8, [r9]
        str     r8, [r5]

        add     r9, r9, #4
        add     r5, r5, #4
        cmp     r5, r7
        blt     1b

        @; adjust the link register
        @; r1 contains the start of where we're running from now.
        ldr     r5, _bootldr_start
        sub     r3, r5, r1
        add     r6, r6, r3

        mov     pc, r6
@;**********************************************************************************

@;***********************************************************************************
        @;  Printing Functions
        @;
@;***********************************************************************************
        @;
        @; ********************************************************************
        @; InitUARTFF - Initialize Serial Communications
        @; ********************************************************************
        @; Following reset, the UART is disabled. So, we do the following:
        @; 1) set the uart gpio lines in their alternate reg state
        @; 2) set the right baud rate (115200) and set to 8 bit/char mode
        @; 3) enable the fifos
        @; 4) turn on the uart
InitUARTFF:
        mov     r6,lr
        ldr     r1,FFUART_BASE
        @;  disable the ffuart just in case
        add     r2,r1,#__OFFSET(FFIER)
        mov     r0,#0x0
        str     r0,[r2]

        @; now we start setting up the uart
        @;  lets set up the baud rate
        @;  baud rate = 14.7456 * 10^6 / (16 * x) = 921600/x
        @;  115200 => x = 8
        ldr     r1,FFUART_BASE
        @;  get acc3ess to baud control registers
        add     r2,r1,#__OFFSET(FFLCR)
        mov     r0,#LCR_DLAB
        str     r0,[r2]
        @;  set the baud rate
        add     r2,r1,#__OFFSET(FFDLL)
        mov     r0,#0x8
        str     r0,[r2]
        add     r2,r1,#__OFFSET(FFDLH)
        mov     r0,#0x0
        str     r0,[r2]
        @;  clear the latch bit and set to 8 bit per character mode
        add     r2,r1,#__OFFSET(FFLCR)
        mov     r0,#LCR_WLS1
        orr     r0,r0,#LCR_WLS0
        str     r0,[r2]
        @;  turn off modem stuff
        add  r2,r1,#__OFFSET(FFMCR)
        @;mov  r0,#0x22 @; turn on auto flow control
        str  r0,[r2]
        @;  enable the fifos  and reset them for good measure
        add     r2,r1,#__OFFSET(FFFCR)
        mov     r0,#FCR_TRFIFOE
        @; orr  r0,r0,#FCR_ITL_32
        orr     r0,r0,#FCR_RESETTF
        orr     r0,r0,#FCR_RESETRF
        str     r0,[r2]


InitUartFFEnable:
        @;  enable the uart
        ldr     r1,FFUART_BASE
        add     r2,r1,#__OFFSET(FFIER)
        mov     r0,#IER_UUE
        str     r0,[r2]

        tst     r12, #BF_SQUELCH_SERIAL
        bne     10f

@; trying to get rid of garbage when the UART turns on
mov     r2, #0
str     r2, [r1, #__OFFSET(FFTHR)]
str     r2, [r1, #__OFFSET(FFTHR)]
str     r2, [r1, #__OFFSET(FFTHR)]
str     r2, [r1, #__OFFSET(FFTHR)]
str     r2, [r1, #__OFFSET(FFTHR)]
str     r2, [r1, #__OFFSET(FFTHR)]

        @; transmit a character or two
        mov     r2, #'\r'
        str     r2, [r1, #__OFFSET(FFTHR)]
        mov     r2, #'\n'
        str     r2, [r1, #__OFFSET(FFTHR)]
        mov     r2, #'U'
        str     r2, [r1, #__OFFSET(FFTHR)]
        mov     r2, #'F'
        str     r2, [r1, #__OFFSET(FFTHR)]
        mov     r2, #'F'
        str     r2, [r1, #__OFFSET(FFTHR)]
        mov     r2, #'\r'
        str     r2, [r1, #__OFFSET(FFTHR)]
        mov     r2, #'\n'
        str     r2, [r1, #__OFFSET(FFTHR)]

10:
        mov     pc, r6          @; All done, return

@; ********************************************************************
@; PrintHexNibble -- prints the least-significant nibble in R0 as a
@; hex digit
@;   r0 contains nibble to write as Hex
@;   r1 contains base of serial port
@;   writes r0 with RXSTAT, modifies r0,r2
@;   Falls through to PrintChar
@; ********************************************************************
PrintHexNibble:
        adr     r2, HEX_TO_ASCII_TABLE
        and     r0, r0, #0xF
        ldrb    r0, [r2,r0]     @; convert to ascii
1:
        ldr     r2,[r1,#__OFFSET(FFLSR)]@; check status
        tst     r2,#LSR_TEMT @; TX empty
        beq     1b
        str     r0,[r1,# __OFFSET(FFTHR)]
        ldr     r0,[r1,#__OFFSET(FFLSR)]
        mov     pc, lr

@; ********************************************************************
@; PrintChar -- prints the character in R0
@;   r0 contains the character
@;   r1 contains base of serial port
@;   writes r0 with UTSR1, modifies r0,r1,r2
@;********************************************************************

PrintChar:
        @; see if printing is disabled
@;        tst     r12, #BF_SQUELCH_SERIAL
@;        movne   pc, lr          @; return if squelched

AlwaysPrintChar:
TXBusy:
        ldr     r2,[r1,#__OFFSET(FFLSR)]@; check status
        tst     r2,#LSR_TEMT @; TX empty
        beq     TXBusy
        str     r0,[r1,# __OFFSET(FFTHR)]
        ldr     r0,[r1,#__OFFSET(FFLSR)]
        mov     pc, lr

@; ********************************************************************
@; PrintWord -- prints the 4 characters in R0
@;   r0 contains the binary word
@;   r1 contains the base of the serial por
@;   writes r0 with RXSTAT, modifies r1,r2,r3,r4
@; ********************************************************************
PrintWord:
        mov     r3, r0
        mov     r4, lr
        bl      PrintChar

        mov     r0,r3,LSR #8    @; shift word right 8 bits
        bl      PrintChar

        mov     r0,r3,LSR #16   @; shift word right 16 bits
        bl      PrintChar

        mov     r0,r3,LSR #24   @; shift word right 24 bits
        bl      PrintChar

        mov     r0, #'\r'
        bl      PrintChar

        mov     r0, #'\n'
        bl      PrintChar

        mov     pc, r4

@; ********************************************************************
@; PrintHexWord -- prints the 4 bytes in R0 as 8 hex ascii characters
@;   followed by a newline
@;   r0 contains the binary word
@;   r1 contains the base of the serial port
@;   Writes r0 with RXSTAT, modifies r1,r2,r3,r4
@; ********************************************************************
PrintHexWord:
        mov     r4, lr
        mov     r3, r0
        mov     r0, r3,LSR #28
        bl      PrintHexNibble
        mov     r0, r3,LSR #24
        bl      PrintHexNibble
        mov     r0, r3,LSR #20
        bl      PrintHexNibble
        mov     r0, r3,LSR #16
        bl      PrintHexNibble
        mov     r0, r3,LSR #12
        bl      PrintHexNibble
        mov     r0, r3,LSR #8
        bl      PrintHexNibble
        mov     r0, r3,LSR #4
        bl      PrintHexNibble
        mov     r0, r3
        bl      PrintHexNibble
        mov     r0, #'\r'
        bl      PrintChar
        mov     r0, #'\n'
        bl      PrintChar
        mov     pc, r4

@;  Utility fxns
@; ********************************************************************
@; bootLinux - boots into another image. DOES NOT RETURN!
@; r0 = must contain a zero or else the kernel loops
@; r1 = architecture type (6 for old kernels, 17 for new)
@; r2 = entry point of kernel
@; ********************************************************************
.globl bootLinux
bootLinux:
        mov     r10, r1
        mov     r11, r2
        @; flush the I+D TLB
        @;  c8 is TLB operation register
        @;  crm = c7 opc2 == 0 indicates Flush I+D TLB
        @;  r0 is ignored
        @; clean data cache
        mov     r0, #0xa0000000
        mov     ip, #1024  @; ip == r12
2:
        mcr     p15, 0, r0, c7, c2, 5           @; allocate line
        add     r0, r0, #32
        subs    ip, ip, #1
        bne     2b

        mcr     p15, 0, r0, c7, c10, 4          @; drain Write Back

        mcr     p15, 0, r0, c8, c7, 0x00        @; invalidate I+D TLB
        cpwait  r0

        mov     r3, #0x120                      @; xscale says p needs to be 0 ???
        mcr     p15, 0, r3, c1, c0, 0           @; disable mmu
        cpwait  r0

        @; be sure pipeline is clear
        mov     r0,r0
        mov     r0,r0
        mov     r0,r0

        @; flush Dcache
        mcr     p15, 0, r0, c7, c6, 0

        @; flush Icache and BTB
        mcr     p15, 0, r0, c7, c5, 0

        @; zero PID in Process ID Virtual Address Map register.
        mov     r0, #0
        mcr     p15, 0, r0, c13, c0, 0
        cpwait  r0

        mov     r0, #0
        mov     r1, r10
        ldr     r2, [r2, #0]
        mov     r2, #0
        mov     pc, r11         @; jump to addr. bootloader is done


        @; ********************************************************************
        @; boot - boots into another image. DOES NOT RETURN!
        @; r0 = pointer to bootinfo
        @; r1 = entry point of kernel
        @; ********************************************************************

        @;  This fxn does not turn off the MMU prior to jumping!  seems like a bug to me  (bavery) !!!
.globl boot
boot:
        @; flush the I+D TLB
        @;  c8 is TLB operation register
        @;  crm=c7 opc2==0 indicates Flush I+D TLB
        @;  r0 is ignored
        mcr     p15, 0, r0, c8, c7, 0x00        @; invalidate I+D TLB
        mcr     p15, 0, r0, c7, c10, 4 @; drain the write buffer
        cpwait r2

        @; make sure the pipeline is emptied
        mov     r0,r0
        mov     r0,r0
        mov     r0,r0
        mov     r0,r0

        mov     pc,r1           @; jump to addr. bootloader is done


.globl asmEnableCaches
asmEnableCaches:
        @; params:      r0 = dcache, r1 = icache
        @; return:      r0,r1,r2,r3  trashed.

        mov     r3,#0
        orr     r3,r0,r3
        orr     r3,r3,r1,lsl #1

        @; get a picture of the current cache situation
        mrc     p15, 0, r2, c1, c0, 0  @; control address

        @; set dcache, clear it if requested
        tst     r3,#(1 << 0)
        orrne   r2,r2,#0x04
        biceq   r2,r2,#0x04 @; bit 2 is dcache

        @; clean data cache
        mov     r0, #0xa0000000
        mov     ip, #1024
2:
        mcr     p15, 0, r0, c7, c2, 5           @; allocate line
        add     r0, r0, #32
        subs    ip, ip, #1
        bne     2b

        @; now flush the dcache
        mcr p15,0,r0,c7,c6,0

        @; also the tlb dcache entries
        mcr p15,0,r0,c8,c6,0

        @; set icache, clear it if requested
        tst     r3,#(1<<1)
        orrne   r2,r2,#0x1000
        biceq   r2,r2,#0x1000  @; bit 12 is icache

        @; flush icache
        mcr     p15, 0, r0, c7,c5,0

        @; also the tlb icache entries
        mcr p15,0,r0,c8,c5,0

        @; store the new values
        mcr     p15, 0, r2, c1, c0, 0
        mov     pc,lr

.globl readPC
readPC:
        mov     r0,pc
        mov     pc,lr

.globl readCPR0
readCPR0:
        mrc     p15,0,r0,c0,c0,0
        mov     pc,lr

.globl readCPR1
readCPR1:
        mrc     p15,0,r0,c1,c0,0
        mov     pc,lr

.globl readCPR2
readCPR2:
        mrc     p15,0,r0,c2,c0,0
        mov     pc,lr

.globl readCPR3
readCPR3:
        mrc     p15,0,r0,c3,c0,0
        mov     pc,lr

@; no CPR4

.globl readCPR5
readCPR5:
        mrc     p15,0,r0,c5,c0,0
        mov     pc,lr

.globl readCPR6
readCPR6:
        mrc     p15,0,r0,c6,c0,0
        mov     pc,lr

.globl readCPR7
readCPR7:
        mrc     p15,0,r0,c7,c0,0
        mov     pc,lr

.globl readCPR8
readCPR8:
        mrc     p15,0,r0,c8,c0,0
        mov     pc,lr

.globl readCPR9
readCPR9:
        mrc     p15,0,r0,c9,c0,0
        mov     pc,lr

.globl readCPR10
readCPR10:
        mrc     p15,0,r0,c10,c0,0
        mov     pc,lr

@; no 11 or 12

.globl readCPR13
readCPR13:
        mrc     p15,0,r0,c13,c0,0
        mov     pc,lr

.globl readCPR14
readCPR14:
        mrc     p15,0,r0,c14,c0,0
        mov     pc,lr

.globl readCPR15
readCPR15:
        mrc     p15,0,r0,c15,c0,0
        mov     pc,lr


.globl writeBackDcache
writeBackDcache:
        @; r0 points to the start of a 16384 byte region of readable
        @; data used only for this cache flushingroutine. If this area
        @; is to be used by other code, then 32K must be loaded and the
        @; flush mcr is not needed.

        @; return:      r0,r1,r2 trashed. data cache is clean
        add r1,r0,#32768
l1:
        ldr r2,[r0],#32
        teq r1, r0
        bne l1
        mcr p15,0,r0,c7,c6,0
        mov pc,lr


.globl flushTLB
flushTLB:
        @; flush the I/D caches
        @;  c7 == cache control operation register
        @;  crm==c7 opc2==0 indicates Flush I+D Cache
        @;  r0 is ignored
        mcr     p15, 0, r0, c7, c7, 0x00        @; invalidate I, D caches & BTB
        @; flush the I+D TLB
        @;  c8 is TLB operation register
        @;  crm=c7 opc2==0 indicates Flush I+D TLB
        @;  r0 is ignored
        mcr     p15, 0, r0, c8, c7, 0x00        @; invalidate I & D TLBs

        mcr     p15, 0, r0, c7, c5, 0           @; invalidate I cache & BTB
        cpwait  r0

        @; invalidate all tlb entries
        mcr     p15, 0, ip, c7, c10, 4          @; Drain Write (& Fill) Buffer
        mcr     p15, 0, ip, c8, c7, 0           @; invalidate I & D TLBs
        @; return
        mov     pc,lr

.globl enableMMU
enableMMU:

        @; r0 = translation table base
        ldr     r0, DW_MMU_TABLE
        mcr     p15, 0, r0, c2, c0, 0

        @; r0 = domain access control
        ldr     r0,MMU_DOMCTRL
        mcr     p15, 0, r0, c3, c0, 0

        @; enable the MMU
        mrc     p15, 0, r0, c1, c0, 0
        orr     r0, r0, #1      @; bit 0 of c1 is MMU enable

        @; make sure Virtual interrupt vector adjust is OFF
        @; bic  r0, r0, #(1<<13)

#ifdef CACHE_ENABLED
        @; orr  r0, r0, #4      @; bit 2 of c1 is DCACHE enable
        @; orr  r0, r0, #8      @; bit 3 of c1 is WB enable
#endif
        mcr     p15, 0, r0, c1, c0, 0
        cpwait  r0
        @; flush the I/D caches
        @;  c7 == cache control operation register
        @;  crm==c7 opc2==0 indicates Flush I+D Cache
        @;  r0 is ignored
        @; mcr  p15, 0, r0, c7, c7, 0x00
        @; flush the I+D TLB
        @;  c8 is TLB operation register
        @;  crm=c7 opc2==0 indicates Flush I+D TLB
        @;  r0 is ignored
        @; mcr  p15, 0, r0, c8, c7, 0x00        @; invalidate I & D TLBs
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        mov     pc, lr


@; STUBS
.globl SleepReset_Resume
SleepReset_Resume:
        mov     pc,lr


@; ********************************************************************
@; Data Area
@; ********************************************************************

@;        .align 2
@;CRC32POLY:
@;        .word   0x04c11db7

        .align  2
addr_start:
        .word   ResetEntryPoint

        .align  2
.globl HEX_TO_ASCII_TABLE
HEX_TO_ASCII_TABLE:
        .ascii  "0123456789ABCDEF"

        .align  2
STR_NEWLINES:
        .ascii  "\r\n\r\n"

        .align  2
STR_WAKE:
        .ascii  "Wake"

        .align  2
DW_STACK_START:
        .word   STACK_BASE+STACK_SIZE-16

        .align  2
DW_MMU_TABLE:
        .word   MMU_TABLE_START

        .align  2
DW_CACHE_FLUSH_REGION:
        .word   CACHE_FLUSH_BASE

        .align  2
MMU_DOMCTRL: @; MMU domain control
        .word   0xFFFFFFFF

        .align  2
STR_MTST:
        .ascii  "MTST"
        .align  2
STR_ENDM:
        .ascii  "ENDM"
        .align  2
STR_UNDEF:
        .ascii  "UNDF"
        .align  2
STR_SWI:
        .ascii  "SWI "

        .align  2
STR_PREFETCH_ABORT:
        .ascii  "PABT"

        .align  2
STR_DATA_ABORT:
        .ascii  "DABT"
        .word   0

        .align  2
STR_IRQ:
        .ascii  "IRQ "

        .align  2
STR_FIQ:
        .ascii  "FIQ "

        .align  2
STR_NOT_USED:
        .ascii  "NUSD"
        .align  2
STR_DUCK:
        .ascii  "DUCK"

        .align  2
STR_RELO:
        .ascii  "RELO"
STR_FROM:
        .ascii  "from"
STR_TO:
        .ascii  "to  "

STR_STEP:
        .ascii  "_STP"

        .globl boot_flags_ptr
boot_flags_ptr:
        .word   STACK_BASE+STACK_SIZE-4

.globl SerBase
SerBase:
        .long __BASE(FFUART)

        .align  2
STR_GPIO:
        .ascii  "GPIO"

        .align  2
STR_MEMREGS:
        .ascii  "MEMR"

        @;  ******************************************************************
.align 4
MEM_BASE:
        .long   __BASE(MDCNFG)
OSCR_BASE:
        .long   __BASE(OSCR)
CLOCK_BASE:
        .long   __BASE(CCCR)
GPIO_BASE:
        .long __BASE(GPLR0)
IRQ_BASE:
        .long __BASE(ICIP)
FFUART_BASE:
        .long __BASE(FFUART)
PMCR_BASE:
        .long __BASE(PMCR)
CCCR_INIT:
        .long ((1 << 25) | (6 << 7) | (1 << 4))
CKEN_INIT:
        .long (0 << 23) | (1 << 22) | (1 << 20) | (0 << 13) | (1 << 9) | (1 << 6) | (0 << 3) | (1 << 1);
OSCC_INIT:
        .long 0x00000007
MDREFR_STEP_5_INIT_A:
        .long (MDREFR_K0DB4 | MDREFR_K0RUN | MDREFR_SLFRSH | (0x31))
@; SXCNFG_INIT:
@;         .long 0x00002013
MDCNFG_STEP_1d_INIT:  @;  set up but not enabled !!!
        .long (MDCNFG_DCAC0 | (2*MDCNFG_DRAC0) | MDCNFG_DNB0 | (2*MDCNFG_DTC0) \
                | MDCNFG_DLATCH0 | (2*MDCNFG_DCAC2) | (2*MDCNFG_DRAC2) \
                | MDCNFG_DNB2 | (2*MDCNFG_DTC2) | MDCNFG_DLATCH2)

@;                3322222222221111111111
@;                10987654321098765432109876543210
@;                --------------------------------
@;                00001010110100000000101011001000
@;                   0   a   d   0   0   a   c   8
@;                00001010110010000000101011001000
@;             HTC:  0   a   c   8   0   a   c   8

HX4700_CheckActionButton:
        ldr r0, =0x40e00008
        ldr r0, [r0]
        tst r0, #0x40000000 @; GPIO 94
        beq 2f

        ldr r0, =0x40e00100
        ldr r0, [r0]
        tst r0, #8          @; GPIO 99

        bicne r12, r12, #BF_NORMAL_BOOT  @; They're both pressed

2:
        orr     r12, r12, #BF_BUTTON_CHECKED
        mov     pc, lr

w1_write_byte:
@; clobers r1, r2
@; Returns read value in r0
        ldr r1, =0x0c000600
1:
        ldrh r2, [r1, #4]       @; IRQ Status
        tst r2, #(1 << 4)       @; Test for RBF set
        beq 2f                  @; branch if RBF not set
        ldrh r2, [r1, #2]       @; TXRX Register (Make sure it's empty)
        b 1b

2:

        @; We are certain the RBF is clear now

        strh r0, [r1, #2]

1:
        ldrh r2, [r1, #4]       @; IRQ Status
        tst r2, #(1 << 4)       @; Test for RBF set
        beq 1b                  @; branch if not set

        ldrh r0, [r1, #8]       @; gratuitous delay
        ldrh r0, [r1, #2]       @; TXRX Register
        mov pc, lr

w1_read_byte:
        ldr r0, =0xff
        b w1_write_byte

battery_current:
        mov r6, lr

        mov r0, #100
        bl Smsleep

        @; ************ beginning if w1_init **************
        @; Turn on EX0, EX1, and OWM clocks
        ldr r0, =0x0c000500
        ldr r1, =(1 << 13) | (1 << 14) | (1 << 3)
        ldrh r2, [r0]
        orr r2, r2, r1
        strh r2, [r0]

        mov r0, #1
        bl Smsleep

        ldr r0, =0x0c000880
        ldrh r3, [r0]
        bic r3, r3, #(1 << 5) @; Clear OWM_SMB
        orr r3, r3, #(1 << 4) @; Set OWM_EN
        strh r3, [r0]

        mov r0, #1
        bl Smsleep

1:
        @; Set bit 6 of EXTCF Reset (reset owm)
        ldr r0, =0x0c000882
        ldrh r3, [r0]
        @;orr r3, r3, #(1 << 6)
        orr r3, r3, #0x40
        strh r3, [r0]

        mov r0, #1
        bl Smsleep

        @; Clear bit 6 of EXTCF Reset (reset owm)
        ldr r0, =0x0c000882
        @;ldrh r3, [r0]
        @;bic r3, r3, #(1 << 6)
        mov r3, #0
        strh r3, [r0]

        mov r0, #1
        bl Smsleep

        ldr r0, =0x0c000608     @; CLOCK_DIV
        ldr r1, =2
        strh r1, [r0]

        mov r0, #10
        bl Smsleep

        @; ************ end if w1_init **************


        @; ***************** beginning of w1_detect
        ldr r3, =0x10

detect_again:

        cmp r4, #0

        moveq r0, #0
        bne keep_going

ldr r1,SerBase
mov r0, #'X'
bl PrintChar
        ldr r0, =0xdeadbeef
        mov pc, r6           @; We weren't able to find it.

keep_going:

        sub r4, r4, #1

        @; delay first to make trying again easier
        mov r0, #1
        bl Smsleep

        ldr r5, =0x0c000600     @; CMD
        mov r0, #1
        strh r0, [r5]           @; Reset 1 wire

        mov r0, #1
        bl Smsleep

97:
        ldrh r0, [r5, #4]       @; IRQ Status
        tst r0, #(1 << 0)       @; Check PD bit.  Set when PD is done.
        bne go_on

        ldr r1, SerBase
        bl PrintHexWord

        b 97b

go_on:

        @; The PD bit is set.
        tst r0, #(1 << 1)       @; Check PDR bit.
        bne detect_again        @; Branch if it wasn't detected to try again

        ldr r0, =10
        bl Smsleep
        @; ***************** end of w1_detect

        ldr r0, =0xcc           @; skip net address
        bl w1_write_byte
        ldr r0, =0x69           @; read
        bl w1_write_byte
        ldr r0, =0x0e           @; address
        bl w1_write_byte

        bl w1_read_byte
        mov r3, r0, lsl #8

        bl w1_read_byte
        orr r0, r3, r0

        mov r0, r0, lsr #3

        mov pc, r6

Smsleep:
        mov r3, lr
        @; clobbers r0, r1, r2
        @; *((volatile unsigned int*)0x40a000c4) = 0x82;
        ldr r1, =0x40a000c4
        mov r2, #0x82
        str r2, [r1]
        @; *((volatile unsigned int*)0x40a00084) = milliseconds;
        sub r1, r1, #0x40
        str r0, [r1]

        @; *((volatile unsigned int*)0x40a00044) = 0;
        sub r1, r1, #0x40
        mov r2, #0
        str r2, [r1]

        @; while(*((volatile unsigned int*)0x40a00044) < milliseconds);
1:
        ldr r2, [r1]
        cmp r2, r0
        blt 1b

        mov pc, r3

@; vim600: set comments=:@; :
