/****************************************************************************/
/* Copyright 2002 Compaq Computer Corporation.                              */
/*                                           .                              */
/* Copying or modifying this code for any purpose is permitted,             */
/* provided that this copyright notice is preserved in its entirety         */
/* in all copies or modifications.  COMPAQ COMPUTER CORPORATION             */
/* MAKES NO WARRANTIES, EXPRESSED OR IMPLIED, AS TO THE USEFULNESS          */
/* OR CORRECTNESS OF THIS CODE OR ITS FITNESS FOR ANY PARTICULAR            */
/* PURPOSE.                                                                 */
/****************************************************************************/
/*
 * flashif.c - Low level flash interface routines
 *
 */

#include "btflash.h"

#include "params.h"

#define FLASH_TIMEOUT 20000000

/* default to physical address, update this variable when MMU gets changed */
volatile unsigned long *flashword = (volatile unsigned long*)FLASH_BASE;
#define bothbanks(w_) ((((w_)&0xFFFF) << 16)|((w_)&0xFFFF))


/*
 * XXX ugly hack.  we'll work this out to be more elegan later...
 */

int  bt_flash_organization = BT_FLASH_ORGANIZATION_2x16;

int
flash_addr_shift(
    void)
{
    switch (bt_flash_organization) {
    case BT_FLASH_ORGANIZATION_2x16:
        return (2);
        break;

    case BT_FLASH_ORGANIZATION_1x16:
        return (1);
        break;

    default:
        putLabeledWord("flash_addr_shift(): "
                       "bad bt_flash_organization=",
                       bt_flash_organization);
        return (0);
        break;
    }

    return (-1);
}

unsigned long
flash_make_val(
    unsigned long   inval)
{
    switch (bt_flash_organization) {
    case BT_FLASH_ORGANIZATION_2x16:
        return (bothbanks(inval));
        break;

    case BT_FLASH_ORGANIZATION_1x16:
        return (inval);
        break;

    default:
        putLabeledWord("flash_make_val(): "
                       "bad bt_flash_organization=",
                       bt_flash_organization);
        return (0);
        break;
    }

    return (-1);
}

#ifdef __ARM_ARCH_3__
#error 16 bit flash interface does not work with armv3 instructions
#endif

void
flash_write_cmd(
    unsigned    cmd_addr,
    int         cmd)
{
#if 0
putLabeledWord("flash_write_cmd_address: ", (unsigned long)cmd_addr);
putLabeledWord("               _cmd    : ", (unsigned long)cmd);
return;
#endif
#if 0
  putLabeledWord("flash_write_cmd: cmd_addr=", cmd_addr);
  putLabeledWord("        cmd=", cmd);
  putLabeledWord("        org=", bt_flash_organization);
  putLabeledWord("        flashword=", flashword);
#endif
    switch (bt_flash_organization) {
    case BT_FLASH_ORGANIZATION_2x16:
        flashword[cmd_addr] = bothbanks(cmd);
        break;

    case BT_FLASH_ORGANIZATION_1x16:
        ((short*)flashword)[cmd_addr] = cmd;
        break;

    default:
        putLabeledWord("flash_write_cmd(): bad bt_flash_organization=",
                       bt_flash_organization);
        break;
    }
}

unsigned long
flash_read_val(unsigned addr)
{
    unsigned long retval = 0;

    switch (bt_flash_organization) {
    case BT_FLASH_ORGANIZATION_2x16:
        retval = flashword[addr];
        break;

    case BT_FLASH_ORGANIZATION_1x16:
        retval = ((unsigned short*)flashword)[addr];
        break;

    default:
        putLabeledWord("flash_read_val(): bad bt_flash_organization=",
                       bt_flash_organization);
        break;
    }

    return (retval);
}

unsigned long
flash_read_array(
    unsigned    addr)
{
    char*   dst;

    /* force read array mode */
    flash_write_cmd(0x55, 0xff);

    switch (bt_flash_organization) {
    case BT_FLASH_ORGANIZATION_2x16:
        dst = (char*)flashword + (addr & ~0x3);
        return (*((unsigned long*)dst));

    case BT_FLASH_ORGANIZATION_1x16:
        dst = (char*)flashword + (addr & ~0x1);
#if 0
        putLabeledWord("fra(1x16), addr=0x", (unsigned long)dst);
#endif
        return (*((unsigned short*)dst));
        break;

    default:
        putLabeledWord("flash_read_array(): bad bt_flash_organization=",
                       bt_flash_organization);
        break;
    }
    return 0;
}

void
flash_write_val(
    unsigned    addr,
    int         val)
{
    char*   dst;

putLabeledWord("flash_write_address: ", (unsigned long)addr);
putLabeledWord("           _value  : ", (unsigned long)val);
return;

    switch (bt_flash_organization) {
    case BT_FLASH_ORGANIZATION_2x16:
        dst = (char*)flashword + (addr & ~0x3);
        *((unsigned long*)dst) = bothbanks(val);
        break;

    case BT_FLASH_ORGANIZATION_1x16:
        dst = (char*)flashword + (addr & ~0x1);
        *((unsigned short*)dst) = val;
        break;

    default:
        putLabeledWord("flash_write_val(): bad bt_flash_organization=",
                       bt_flash_organization);
        break;
    }
}

int
is_flash_16bit(void)
{
    return 0;
}

/* remove remaining direct accesses to flashword */
void nflash_write_cmd(unsigned long addr, unsigned long val)
{
#if 0
putLabeledWord("nflash_write_cmd_addr: ", (unsigned long)addr);
putLabeledWord("                _val : ", (unsigned long)val);
putLabeledWord("              Actual : ", (unsigned long)&flashword[addr]);
#endif
    flashword[addr] = bothbanks(val);
}
void nflash_write_val( unsigned addr, int val)
{
#if 0
putLabeledWord("nflash_write_val_addr: ", (unsigned long)addr);
putLabeledWord("                _val : ", (unsigned long)val);
putLabeledWord("      Actual Address :", &flashword[addr]);
#endif
extern int print_addr;
if(print_addr) {
    putLabeledWord("Write@", (unsigned long)&flashword[addr]);
}
    flashword[addr] = val;
}
unsigned long nflash_read_val( unsigned addr)
{
#if 0
putLabeledWord("nflash_read_val: cmd_addr=", addr);
putLabeledWord("                      org=", bt_flash_organization);
putLabeledWord("                flashword=", flashword);
putLabeledWord("           Actual Address=", &flashword[addr]);
#endif
    return flashword[addr];
}
/* short read/write */
void flash_write_val_short( unsigned addr, int val)
{
    unsigned short* flashshort = (unsigned short*)flashword;
putLabeledWord("flash_write_val_short: ", (unsigned long)addr);
putLabeledWord("               _val  : ", (unsigned long)val);
return;
    flashshort[addr] = val;
}
unsigned long flash_read_val_short( unsigned addr)
{
    unsigned short* flashshort = (unsigned short*)flashword;
    return flashshort[addr];
}
