#include "bootldr.h"
#include "lcd.h"
#include "mach_type.h"


#define FRAME_BUFFER_BASE 0x08800000

enum lcd_type lcd_type = LCDP_end_sentinel;

static unsigned short *lcd_image_buffer;

void
lcd_display_bitmap(char *bitmap_in, size_t len, int params_id, lcd_params_t *params)
{
    int             num_pixels;
    unsigned short  pixel;
    unsigned short *bufp = (unsigned short*)FRAME_BUFFER_BASE;
    unsigned short *bmp  = (unsigned short*)bitmap_in;

    for(num_pixels = LCD_NUM_PIXELS(); num_pixels; num_pixels--) {
        pixel   = *bmp++;
        *bufp++ = pixel;
    }
}

void lcd_on(void)
{
    *(volatile unsigned int *)0x40c00000 = 0x00000001;   /* Set up PWM to control backlight */
    *(volatile unsigned int *)0x40c00004 = 0x0000003f;   /* Set up PWM to control backlight */
    *(volatile unsigned int *)0x40c00008 = 0x000000c8;   /* Set up PWM to control backlight */
}

void
lcd_off(enum lcd_type params_id)
{
#if 0
    *(volatile unsigned int *)0x40e00028 = 0x48000000;   /* Clear GPIO 62 (LCD_RESET#) and 59 (LCD_PC1) */
    msleep(10);
    *(volatile unsigned int *)0x40e00124 = 0x00010000;   /* Clear GPIO 112 (N 2.7v, 7.3v) */
    msleep(10);
    *(volatile unsigned int *)0x40e00124 = 0x00008000;   /* Clear GPIO 111 (AVDD 3.3v) */
    msleep(10);
    *(volatile unsigned int *)0x40e00124 = 0x00004000;   /* Clear GPIO 110 (LVDD 3.3v) */
#endif

    *(volatile unsigned int *)0x40c00000 = 0x00000001;   /* Set up PWM to control backlight */
    *(volatile unsigned int *)0x40c00004 = 0x00000000;   /* Set up PWM to control backlight */
    *(volatile unsigned int *)0x40c00008 = 0x000000c8;   /* Set up PWM to control backlight */
}


void
lcd_light(int on_off, int level)
{
}

void
lcd_fill(int color, int num)
{
    unsigned short* bufp = (unsigned short*)lcd_get_image_buffer();
    int     num_pixels;

    num = (num == 0) ? (LCD_XRES * LCD_YRES) : num;

    for(num_pixels = 0; num_pixels < num ; num_pixels++) {
        *bufp = color;
        bufp++;
    }
}

char*
lcd_get_image_buffer(void)
{
    return (char *)lcd_image_buffer;
}

size_t
lcd_get_image_buffer_len(void)
{
    return (LCD_FB_MAX());
}

void
lcd_init(int mach_type)
{
    void lcd_init2(void);
    /* First, turn the lcd power off, put it in reset. */
    /* This would be unnecessary if you already know it's off. */
    *(volatile unsigned int *)0x40e0002c = 0x40;         /* Clear GPIO 70, LCD_SLIN1 */
    *(volatile unsigned int *)0x40e00124 = 0x0000c000;   /* Clear GPIO 110 (LVDD 3.3v), 111 (AVDD 3.3v) */
    *(volatile unsigned int *)0x40e00028 = 0x40000000;   /* Clear GPIO 62 (LCD_RESET#) */
    msleep(10);
    *(volatile unsigned int *)0x40e00028 = 0x08000000;   /* Clear GPIO 59 (LCD_PC1) */
    *(volatile unsigned int *)0x40e00124 = 0x00010000;   /* Clear GPIO 112 (N 2.7v, 7.3v) */
    msleep(20);
    *(volatile unsigned int *)0x40e00118 = 0x00004000;   /* Set GPIO 110 (LVDD 3.3v) */
    msleep(5);
    *(volatile unsigned int *)0x40e00118 = 0x00008000;   /* Set GPIO 111 (AVDD 3.3v) */

    lcd_init2();

    *(volatile unsigned int *)0x40e00020 = 0x00000040;   /* Set GPIO 70 (LCD_SLIN1) */
    msleep(10);
    *(volatile unsigned int *)0x40e0001c = 0x40000000;   /* Set GPIO 62: LCD_RESET# off */
    msleep(10);
    *(volatile unsigned int *)0x40e0001c = 0x08000000;   /* Set GPIO 59 (LCD_PC1) */
    msleep(10);

    *(volatile unsigned int *)0x40e00118 = 0x00010000;   /* Set GPIO 112 (N 2.7v, 7.3v)  */

}

void
lcd_init2(void)
{
    int i;

    lcd_image_buffer = (unsigned short *)FRAME_BUFFER_BASE;

    /* Reset the ATI device */
    *(volatile unsigned int *)0x40e00028 = 0x20000000;   /* Clear GPIO 61: W3220 Reset */
    msleep(10);
    *(volatile unsigned int *)0x40e0001c = 0x20000000;   /* Set GPIO 61 */
    msleep(10);

#if 0
*(volatile unsigned int *)0x40c00000 = 0x00000001;   /* Set up PWM to control backlight */
*(volatile unsigned int *)0x40c00004 = 0x0000003f;   /* Set up PWM to control backlight */
*(volatile unsigned int *)0x40c00008 = 0x000000c8;   /* Set up PWM to control backlight */

*(volatile unsigned int *)0x40e0002c = 0x40;         /* Clear GPIO 70 */
*(volatile unsigned int *)0x40e00124 = 0x0001c000;   /* Clear GPIO 110, 111, and 112 */
*(volatile unsigned int *)0x40e00028 = 0x48000000;   /* Clear GPIO 62 and 59 */
msleep(10);

*(volatile unsigned int *)0x40e00118 = 0x0000c000;   /* Set GPIO 110-111 */
msleep(10);
#endif

    while(*((volatile unsigned int *)0x08010000) != 0x57441002) {
        msleep(1);
    }

    /* Initialize ATI Device */
    *(volatile unsigned int *)0x08010008 = 0x03157ff0;   // WRAP_BUF_A
    *(volatile unsigned int *)0x08010018 = 0x0c180500;   // CIF_CNTL
    *(volatile unsigned int *)0x08010020 = 0x001c0930;   // CIF_IO
    *(volatile unsigned int *)0x08010028 = 0x00002280;   // CIF_WRITE_DBG
    *(volatile unsigned int *)0x08010024 = 0x00038223;   // CIF_READ_DBG

    *(volatile unsigned int *)0x08010080 = 0x0000003f;   // CLK_PIN_CNTL
    *(volatile unsigned int *)0x08010084 = 0x1F120D01;   // PLL_REF_FB_DIV
#if 1
    *(volatile unsigned int *)0x08010088 = 0x4b00d200;
#else
    *(volatile unsigned int *)0x08010088 = 0000000200
        | (0 << 14) | (0 << 10);        // PLL_CNTL
#endif
    *(volatile unsigned int *)0x0801008c = 0x00000b01;   // SCLK_CNTL
    *(volatile unsigned int *)0x08010090 = 0x00000051;   // PCLK_CNTL
    *(volatile unsigned int *)0x08010094 = 0x00e63001;   // CLK_TEST_CNTL
    *(volatile unsigned int *)0x08010098 = 0x00ff4000;   // PWRMGMT_CNTL
#if 0
    *(volatile unsigned int *)0x0801009c = 0x0000006e;   // PWRMGMT_STATUS
#endif
#if 0
    *(volatile unsigned int *)0x08010144 = 0x00002440;   // RBBM_CNTL
#endif
    *(volatile unsigned int *)0x08010180 = 0x00100006;   // MEM_CNTL
    *(volatile unsigned int *)0x08010188 = 0x15ff1000;   // MC_FB_LOCATION
    *(volatile unsigned int *)0x0801018c = 0x09640011;   // MEM_EXT_CNTL
    *(volatile unsigned int *)0x08010190 = 0xffff8000;   // MC_EXT_MEM_LOCATION
    *(volatile unsigned int *)0x08010194 = 0x1A001545;   // MEM_EXT_TIMING_CNTL
    *(volatile unsigned int *)0x08010198 = 0x00200021;   // MEM_SDRAM_MODE_REG
    msleep(10);
    *(volatile unsigned int *)0x08010198 = 0x80200021;
    msleep(10);
    *(volatile unsigned int *)0x08010198 = 0x00600021;
    msleep(10);
    *(volatile unsigned int *)0x0801019c = 0x7ddd7333;   // MEM_IO_CNTL
    *(volatile unsigned int *)0x08010200 = 0x00000010;   // GEN_INT_CNTL
#if 0
    *(volatile unsigned int *)0x080101a0 = 0x00000100;  // mmMC_EXT_BLK_CNTL
    *(volatile unsigned int *)0x080101a4 = 0x00007800;  // mmMC_BIST_CNTL
    *(volatile unsigned int *)0x080101a8 = 0x00000000;  // mmMC_BIST_COLLAR_READ
    *(volatile unsigned int *)0x080101ac = 0x00000000;  // mmMC_MISMATCH
    *(volatile unsigned int *)0x080101b0 = 0x00000000;  // mmMC_PERF_MON_CNTL
    *(volatile unsigned int *)0x080101b4 = 0x00000000;  // mmMC_PERF_COUNTERS
    *(volatile unsigned int *)0x080101bc = 0x7FFF4000;  // mmMC_BM_MEM_LOCATION
    *(volatile unsigned int *)0x080101c0 = 0x203F2000;  // mmMC_ENHIO_MEM_LOC
#endif
    *(volatile unsigned int *)0x08010410 = 0x00000003;   // LCD_FORMAT
    *(volatile unsigned int *)0x08010414 = 0x079e1c06;   // GRAPHIC_CNTL
    *(volatile unsigned int *)0x08010418 = 0x00800000;   // GRAPHIC_OFFSET
    *(volatile unsigned int *)0x0801041c = 0x000003c0;   // GRAPHIC_PITCH
    *(volatile unsigned int *)0x08010420 = 0x028f01ff;   // CRTC_TOTAL
    *(volatile unsigned int *)0x08010424 = 0x01ef000f;   // ACTIVE_H_DISP
    *(volatile unsigned int *)0x08010428 = 0x02880008;   // ACTIVE_V_DISP
    *(volatile unsigned int *)0x0801042c = 0x01ef000f;   // GRAPHIC_H_DISP
    *(volatile unsigned int *)0x08010430 = 0x02880008;   // GRAPHIC_V_DISP
    *(volatile unsigned int *)0x0801048c = 0x0;          // CRTC_SS
    *(volatile unsigned int *)0x08010490 = 0xa1ff01f9;   // CRTC_LS
    *(volatile unsigned int *)0x08010494 = 0x00000000;   // CRTC_REV
    *(volatile unsigned int *)0x0801049c = 0x80000000;   // CRTC_DCLK
    *(volatile unsigned int *)0x080104a0 = 0xc0000000;   // CRTC_GS
    *(volatile unsigned int *)0x080104a4 = 0x0000028f;   // CRTC_VPOS_GS
    *(volatile unsigned int *)0x080104a8 = 0x040a0104;   // CRTC_GCLK
    *(volatile unsigned int *)0x080104ac = 0x00000000;   // CRTC_GOE
    *(volatile unsigned int *)0x080104b0 = 0x00000000;   // CRTC_FRAME
    *(volatile unsigned int *)0x080104b4 = 0x00000000;   // CRTC_FRAME_POS
    *(volatile unsigned int *)0x080104b8 = 0xDFE00100;   // GPIO_DATA
    *(volatile unsigned int *)0x080104bc = 0xFFFF0000;   // GPIO_CNTL1
    *(volatile unsigned int *)0x080104c4 = 0x00000000;   // LCDD_CNTL1
    *(volatile unsigned int *)0x080104c8 = 0x0003ffff;   // LCDD_CNTL2
    *(volatile unsigned int *)0x080104cc = 0x00abf003;   // GENLCD_CNTL1
    *(volatile unsigned int *)0x080104d0 = 0x00000003;   // GENLCD_CNTL2
    *(volatile unsigned int *)0x080104d4 = 0x00000000;   // DISP_DEBUG
    *(volatile unsigned int *)0x080104e0 = 0x00000000;   // CRTC_DEFAULT_COUNT
    *(volatile unsigned int *)0x080104e4 = 0x00000000;   // LCD_BACKGROUND_COLOR
    *(volatile unsigned int *)0x080104f0 = 0x00000000;   // CRTC_PS1_ACTIVE
    *(volatile unsigned int *)0x08010524 = 0x000102aa;   // GENLCD_CNTL3
    *(volatile unsigned int *)0x08010538 = 0x00800000;   // DISP_DEBUG2

    *(volatile unsigned int *)0x08010a00 = 0x50413e01;   // EXT_MEM_BW
    *(volatile unsigned int *)0x08010a04 = 0x00000000;   // OFFSET
    *(volatile unsigned int *)0x08010a08 = 0x00043f7f;   // EXT_TIMING_CTL
    *(volatile unsigned int *)0x08010a0c = 0x00000010;   // EXT_CNTL
    *(volatile unsigned int *)0x08010a10 = 0x00250000;   // MODE_REG
    *(volatile unsigned int *)0x08010a18 = 0x0fff0000;   // IO_CNTL
    *(volatile unsigned int *)0x08010a1c = 0x08301480;   // CONFIG
#if 0
    *(volatile unsigned int *)0x08010a20 = 0x00008021;   // CONFIG
    *(volatile unsigned int *)0x08010a24 = 0;
    *(volatile unsigned int *)0x08010a28 = 0;
    *(volatile unsigned int *)0x08010a2c = 0;
    *(volatile unsigned int *)0x08010a30 = 0;
    *(volatile unsigned int *)0x08010a34 = 0;
#endif
    /* End of ATI Init */
    msleep(100);

#if 0
*(volatile unsigned int *)0x40e00020 = 0x00000040;   /* Set GPIO 70 */
msleep(10);
*(volatile unsigned int *)0x40e0001c = 0x40000000;   /* Set GPIO 62: LCD_RESET# off */
msleep(10);
*(volatile unsigned int *)0x40e0001c = 0x08000000;   /* Set GPIO 59 */
msleep(10);
*(volatile unsigned int *)0x40e00118 = 0x00010000;   /* Set GPIO 112 */
msleep(10);
#endif


    {
        int i;

        for(i = 0; i < 640 * 480 / 2; i++) {
            *((volatile unsigned int *)0x08800000 + i) = 0xffffffff;
        }
    }
}



void
lcd_default_init(int params_id, lcd_params_t *params)
{
    lcd_image_buffer = (unsigned short *)FRAME_BUFFER_BASE;
}

void set_pixel_value(unsigned short row, unsigned short col, unsigned short val)
{
    *(lcd_image_buffer + (LCD_XRES * row) + col) = val;
}

unsigned short get_pixel_value(unsigned short row, unsigned short col)
{
    return *(lcd_image_buffer +LCD_XRES * row + col);
}

