#include "bootldr.h"
#include "lcd.h"
#include "mach_type.h"


#define FRAME_BUFFER_BASE 0x08800000

enum lcd_type lcd_type = LCDP_end_sentinel;

static unsigned short *lcd_image_buffer;

void
lcd_display_bitmap(char *bitmap_in, size_t len, int params_id, lcd_params_t *params)
{
    int             num_pixels;
    unsigned short  pixel;
    unsigned short *bufp = (unsigned short*)FRAME_BUFFER_BASE;
    unsigned short *bmp  = (unsigned short*)bitmap_in;

    for(num_pixels = LCD_NUM_PIXELS(); num_pixels; num_pixels--) {
        pixel   = *bmp++;
        *bufp++ = pixel;
    }
}

void lcd_on(void)
{

    putstr("LCD On 2\r\n");
    *(unsigned int *)0x40e0002c = 0x40;         /* Clear GPIO 70 */
    *(unsigned int *)0x40e00028 = 0x40000000;   /* Clear GPIO 62 */
    *(unsigned int *)0x40e00124 = 0x00010000;   /* Clear GPIO 110, 111, and 112 */
    *(unsigned int *)0x40e00124 = 0x00004000;
    *(unsigned int *)0x40e00124 = 0x00008000;
    *(unsigned int *)0x40e00028 = 0x08000000;   /* Clear GPIO 59 */
    msleep(1000);
    *(unsigned int *)0x40e00118 = 0x00004000;   /* Set GPIO 110: LCD_LVDD_3V3_ON */
    msleep(1000);
    *(unsigned int *)0x40e00118 = 0x00008000;   /* Set GPIO 111: LCD_AVDD_3V3_ON */
    msleep(1000);

    /* Initialize ATI Device */
    *(unsigned int *)0x08010080 = 0x000401bf;
    *(unsigned int *)0x08010084 = 0x50500d04;
    *(unsigned int *)0x08010088 = 0x4b000200;
    *(unsigned int *)0x0801008c = 0x00000b11;
    *(unsigned int *)0x08010090 = 0x000000b1;
    *(unsigned int *)0x08010094 = 0x00000001;
    *(unsigned int *)0x08010098 = 0xffff11c5;
    *(unsigned int *)0x08010188 = 0x15ff1000;
    *(unsigned int *)0x08010410 = 0x00000003;
    *(unsigned int *)0x08010414 = 0x01d41c06;
    *(unsigned int *)0x08010418 = 0x00100000;
    *(unsigned int *)0x0801041c = 0x000001e0;
    *(unsigned int *)0x08010420 = 0x014f010f;
    *(unsigned int *)0x08010424 = 0x00f00000;
    *(unsigned int *)0x08010428 = 0x014f000f;
    *(unsigned int *)0x0801042c = 0x00f00000;
    *(unsigned int *)0x08010430 = 0x014f000f;
    *(unsigned int *)0x0801048c = 0x80150014;
    *(unsigned int *)0x08010490 = 0xa0fb00f7;
    *(unsigned int *)0x08010494 = 0x0000000a;
    *(unsigned int *)0x0801049c = 0x81700030;
    *(unsigned int *)0x080104a0 = 0xc0080007;
    *(unsigned int *)0x080104a4 = 0x00080007;
    *(unsigned int *)0x080104a8 = 0x8015010f;
    *(unsigned int *)0x080104ac = 0x00000000;
    *(unsigned int *)0x080104b0 = 0x00000000;
    *(unsigned int *)0x080104b4 = 0x00000000;
    *(unsigned int *)0x080104c4 = 0x00000000;
    *(unsigned int *)0x080104c8 = 0x0003ffff;
    *(unsigned int *)0x080104cc = 0x00fff003;
    *(unsigned int *)0x080104d0 = 0x00000003;
    *(unsigned int *)0x080104d4 = 0x00000100;
    *(unsigned int *)0x080104e0 = 0x00000000;
    *(unsigned int *)0x080104e4 = 0x0000ff00;
    *(unsigned int *)0x080104f0 = 0x41060010;
    *(unsigned int *)0x08010524 = 0x000102aa;
    *(unsigned int *)0x08010538 = 0x00800000;
    /* End of ATI Init */

    msleep(1000);
    *(unsigned int *)0x40e00020 = 0x00000040;   /* Set GPIO 70: LCD_SLIN1 */
    msleep(1000);
    *(unsigned int *)0x40e0001c = 0x40000000;   /* Set GPIO 62: LCD_RESET# off */
    msleep(1000);
    *(unsigned int *)0x40e0001c = 0x08000000;   /* Set GPIO 59: LCD_PC1 */
    msleep(1000);
    *(unsigned int *)0x40e00118 = 0x00010000;   /* Set GPIO 112: LCD_N2V7_7V3_ON */
    msleep(1000);
}

void
lcd_off(enum lcd_type params_id)
{
}


void
lcd_light(int on_off, int level)
{
}

void
lcd_fill(int color, int num)
{
    unsigned short* bufp = (unsigned short*)lcd_get_image_buffer();
    int     num_pixels;

    num = (num == 0) ? (LCD_XRES * LCD_YRES) : num;

    for(num_pixels = 0; num_pixels < num ; num_pixels++) {
        *bufp = color;
        bufp++;
    }
}

char*
lcd_get_image_buffer(void)
{
    return (char *)lcd_image_buffer;
}

size_t
lcd_get_image_buffer_len(void)
{
    return (LCD_FB_MAX());
}

void lcd_init(int mach_type)
{
    lcd_image_buffer = (unsigned short *)FRAME_BUFFER_BASE;
}

void
lcd_default_init(int params_id, lcd_params_t *params)
{
    lcd_image_buffer = (unsigned short *)FRAME_BUFFER_BASE;
}

void set_pixel_value(unsigned short row, unsigned short col, unsigned short val)
{
    *(lcd_image_buffer + (LCD_XRES * row) + col) = val;
}

unsigned short get_pixel_value(unsigned short row, unsigned short col)
{
    return *(lcd_image_buffer +LCD_XRES * row + col);
}

